package session

import (
	"fmt"
	"html/template"
	"log"
	"net/http"
	"time"

	"github.com/google/uuid"
	"streifling.com/jason/sicherheitsunterweisung/packages/data"
)

func HandleInternalLogin(ss *[]*Session, cs chan<- *Session, db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		instructors, err := db.GetInstructors()
		if err != nil {
			http.Error(w, "HandleInternalLogin: db.GetInstructors(): "+fmt.Sprint(err), http.StatusInternalServerError)
			log.Panicln(err)
		}

		for _, i := range instructors {
			if r.PostFormValue("login") == fmt.Sprint(i.ID) {
				session := new(Session)
				session.ID = uuid.New()
				session.Briefing = new(data.Briefing)
				session.Briefing.InstructorID = i.ID
				(*ss) = append((*ss), session)
				cs <- session

				displayTable(w, db)
				return
			}
		}
		template.Must(template.ParseFiles("templates/login.html")).ExecuteTemplate(w, "content", nil)
	}
}

func HandleSearch(db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		bs, err := db.GetOverviewTableDataByName(r.PostFormValue("search"))
		if err != nil {
			http.Error(w, "DisplayResults: db.ReadByName(r.PostFormValue()): "+fmt.Sprint(err), http.StatusInternalServerError)
		}
		template.Must(template.ParseFiles("templates/table.html")).ExecuteTemplate(w, "rows", bs)
	}
}

func (s *Session) HandleNewBriefing() http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		data := new(briefingHTMLData)
		data.SessionID = s.ID
		template.Must(template.ParseFiles("templates/briefing.html")).ExecuteTemplate(w, "content", data)
	}
}

func (s *Session) HandleNewParticipant(cp chan<- *data.Participant) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var err error
		p := new(data.Participant)
		p.Login, err = generateLogin()
		if err != nil {
			http.Error(w, "AddParticipant: generateLogin(): "+fmt.Sprint(err), http.StatusInternalServerError)
		}
		s.Participants = append(s.Participants, p)
		cp <- p

		data := new(briefingHTMLData)
		data.SessionID = s.ID
		data.Login = p.Login
		if err != nil {
			http.Error(w, "AddParticipant: generateLogin(): "+fmt.Sprint(err), http.StatusInternalServerError)
		}

		template.Must(template.ParseFiles("templates/briefing.html")).ExecuteTemplate(w, "new", data)
	}
}

func (s *Session) HandleBriefingForm(db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		now := time.Now()
		var err error

		s.Briefing.Date = now.Format("2006-01-02")
		s.Briefing.Time = now.Format("15:04:05")
		s.Briefing.Location = r.PostFormValue("location")
		s.Briefing.DocumentName = r.PostFormValue("document")
		s.Briefing.AsOf = r.PostFormValue("as-of")

		err = db.WriteBriefing(s.Briefing)
		if err != nil {
			http.Error(w, "SubmitBriefingForm: db.WriteBriefing(): "+fmt.Sprint(err), http.StatusInternalServerError)
			log.Panicln(err)
		}

		displayTable(w, db)
	}
}

func HandleExternalLogin(ss *[]*Session) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		session, participant, loginCorrect := findCorrectLogin(r.PostFormValue("login"), ss)
		if loginCorrect {
			data := new(participantHTMLData)
			data.SessionID = session.ID
			data.Login = participant.Login

			template.Must(template.ParseFiles("templates/participant.html")).ExecuteTemplate(w, "content", data)
		} else {
			template.Must(template.ParseFiles("templates/login.html")).ExecuteTemplate(w, "content", nil)
		}
	}
}

func (s *Session) HandleParticipant(p *data.Participant, sq *[]data.Question, db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		p.FirstName = r.PostFormValue("first-" + fmt.Sprint(p.Login))
		p.LastName = r.PostFormValue("last-" + fmt.Sprint(p.Login))
		p.Company = r.PostFormValue("company-" + fmt.Sprint(p.Login))

		err := db.WriteParticipant(p)
		if err != nil {
			http.Error(w, "DisplayQuestion: db.WriteParticipant(): "+fmt.Sprint(err), http.StatusInternalServerError)
			log.Panicln(err)
		}

		data := new(questionHTMLData)
		data.SessionID = s.ID
		data.Login = p.Login
		data.Question = (*sq)[0]
		data.QuestionID = 1

		template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
	}
}

func (s *Session) HandleAnswer(db *data.DB, p *data.Participant, sq *[]data.Question, i int64) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		log.Println(i, len(*sq))
		if i < int64(len(*sq)) {
			if err := handleGivenAnswer(s, p, i-1, r, db); err != nil {
				http.Error(w, "DisplayQuestion: handleGivenAnswer(): "+fmt.Sprint(err), http.StatusInternalServerError)
				log.Panicln(err)
			}

			data := new(questionHTMLData)
			data.SessionID = s.ID
			data.Login = p.Login
			data.Question = (*sq)[i]
			data.QuestionID = i + 1

			template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
		} else {
			if err := handleGivenAnswer(s, p, i-1, r, db); err != nil {
				http.Error(w, "DisplayTestResults: handleGivenAnswer(): "+fmt.Sprint(err), http.StatusInternalServerError)
				log.Panicln(err)
			}

			givenAnswers, err := db.GetGivenAnswers(s.Briefing.ID, p.ID, s.Questions)
			if err != nil {
				http.Error(w, "DisplayTestResults: db.GetGivenAnswers(): "+fmt.Sprint(err), http.StatusInternalServerError)
				log.Panicln(err)
			}

			data := new(resultHTMLData)
			data.SessionID = s.ID
			data.Login = p.Login
			data.Incorrect = 0

			data.Questions = make([]htmlQuestion, 0)
			for i, q := range s.Questions {
				question := new(htmlQuestion)
				question.Text = q.Text

				question.Answers = make([]htmlAnswer, 0)
				for j, a := range q.Answers {
					answer := new(htmlAnswer)
					answer.Text = a.Text

					if j+1 == q.Correct {
						answer.Correct = true
					} else {
						answer.Correct = false
					}

					if j+1 == givenAnswers[i] {
						answer.Chosen = true
					} else {
						answer.Chosen = false
					}
					question.Answers = append(question.Answers, *answer)
				}
				data.Questions = append(data.Questions, *question)

				if givenAnswers[i] != q.Correct {
					data.Incorrect++
				}
			}

			template.Must(template.ParseFiles("templates/result.html")).ExecuteTemplate(w, "content", data)
		}
	}
}

func (s *Session) HandleRetry(p *data.Participant, sq *[]data.Question) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		data := new(questionHTMLData)
		data.SessionID = s.ID
		data.Login = p.Login
		data.Question = (*sq)[0]
		data.QuestionID = 1

		template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
	}
}
